@@ -19,15 +19,13 @@ import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/ex
 import * as DND from 'resource:///org/gnome/shell/ui/dnd.js';
 import * as Main from 'resource:///org/gnome/shell/ui/main.js';
 import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
+import {DashItemContainer} from 'resource:///org/gnome/shell/ui/dash.js';
 
 import {WorkspaceIndicator} from './workspaceIndicator.js';
 
 const ICON_TEXTURE_SIZE = 24;
 const DND_ACTIVATE_TIMEOUT = 500;
 
-const TOOLTIP_OFFSET = 6;
-const TOOLTIP_ANIMATION_TIME = 150;
-
 const GroupingMode = {
     NEVER: 0,
     AUTO: 1,
@@ -165,14 +163,14 @@ class WindowTitle extends St.BoxLayout {
             this._icon.child = app.create_icon_texture(ICON_TEXTURE_SIZE);
         } else {
             this._icon.child = new St.Icon({
-                icon_name: 'icon-missing',
+                icon_name: 'application-x-executable',
                 icon_size: ICON_TEXTURE_SIZE,
             });
         }
     }
 }
 
-class BaseButton extends St.Button {
+class BaseButton extends DashItemContainer {
     static {
         GObject.registerClass({
             GTypeFlags: GObject.TypeFlags.ABSTRACT,
@@ -186,12 +184,22 @@ class BaseButton extends St.Button {
     }
 
     constructor(perMonitor, monitorIndex) {
-        super({
+        super();
+
+        this._button = new St.Button({
             style_class: 'window-button',
             can_focus: true,
             x_expand: true,
             button_mask: St.ButtonMask.ONE | St.ButtonMask.THREE,
         });
+        this.setChild(this._button);
+
+        this._button.connect('notify::hover', () => {
+            if (this._button.hover)
+                this.showLabel();
+            else
+                this.hideLabel();
+        });
 
         this._perMonitor = perMonitor;
         this._monitorIndex = monitorIndex;
@@ -199,7 +207,7 @@ class BaseButton extends St.Button {
 
         this.connect('notify::allocation',
             this._updateIconGeometry.bind(this));
-        this.connect('clicked', this._onClicked.bind(this));
+        this._button.connect('clicked', this._onClicked.bind(this));
         this.connect('destroy', this._onDestroy.bind(this));
         this.connect('popup-menu', this._onPopupMenu.bind(this));
 
@@ -216,12 +224,6 @@ class BaseButton extends St.Button {
                 this._windowEnteredOrLeftMonitor.bind(this),
                 this);
         }
-
-        this._tooltip = new Tooltip(this, {
-            style_class: 'dash-label',
-            visible: false,
-        });
-        Main.uiGroup.add_child(this._tooltip);
     }
 
     get active() {
@@ -244,6 +246,18 @@ class BaseButton extends St.Button {
         this._updateVisibility();
     }
 
+    showLabel() {
+        const [, , preferredTitleWidth] = this.label_actor.get_preferred_size();
+        const maxTitleWidth = this.label_actor.allocation.get_width();
+        const isTitleFullyShown = preferredTitleWidth <= maxTitleWidth;
+
+        const labelText = isTitleFullyShown
+            ? '' : this.label_actor.text;
+
+        this.setLabelText(labelText);
+        super.showLabel();
+    }
+
     _setLongPressTimeout() {
         if (this._longPressTimeoutId)
             return;
@@ -344,9 +358,9 @@ class BaseButton extends St.Button {
 
     _updateStyle() {
         if (this._isFocused())
-            this.add_style_class_name('focused');
+            this._button.add_style_class_name('focused');
         else
-            this.remove_style_class_name('focused');
+            this._button.remove_style_class_name('focused');
     }
 
     _windowEnteredOrLeftMonitor(_metaDisplay, _monitorIndex, _metaWindow) {
@@ -382,7 +396,6 @@ class BaseButton extends St.Button {
     }
 
     _onDestroy() {
-        this._tooltip.destroy();
     }
 }
 
@@ -395,15 +408,17 @@ class WindowButton extends BaseButton {
         super(perMonitor, monitorIndex);
 
         this.metaWindow = metaWindow;
+        this._unmanaging = false;
         metaWindow.connectObject(
             'notify::skip-taskbar', () => this._updateVisibility(),
             'workspace-changed', () => this._updateVisibility(),
+            'unmanaging', () => (this._unmanaging = true),
             this);
 
         this._updateVisibility();
 
         this._windowTitle = new WindowTitle(this.metaWindow);
-        this.set_child(this._windowTitle);
+        this._button.set_child(this._windowTitle);
         this.label_actor = this._windowTitle.label_actor;
 
         this._contextMenu = new WindowContextMenu(this, this.metaWindow);
@@ -449,6 +464,9 @@ class WindowButton extends BaseButton {
     }
 
     _updateVisibility() {
+        if (this._unmanaging)
+            return;
+
         this.visible = this._isWindowVisible(this.metaWindow);
     }
 
@@ -532,7 +550,7 @@ class AppButton extends BaseButton {
         this._updateVisibility();
 
         let stack = new St.Widget({layout_manager: new Clutter.BinLayout()});
-        this.set_child(stack);
+        this._button.set_child(stack);
 
         this._singleWindowTitle = new St.Bin({
             x_expand: true,
@@ -729,28 +747,24 @@ class WindowList extends St.Widget {
         let box = new St.BoxLayout({x_expand: true, y_expand: true});
         this.add_child(box);
 
-        let layout = new Clutter.BoxLayout({homogeneous: true});
-        this._windowList = new St.Widget({
+        this._windowList = new St.BoxLayout({
             style_class: 'window-list',
             reactive: true,
-            layout_manager: layout,
             x_align: Clutter.ActorAlign.START,
             x_expand: true,
             y_expand: true,
         });
         box.add_child(this._windowList);
 
-        this._windowList.connect('style-changed', () => {
-            let node = this._windowList.get_theme_node();
-            let spacing = node.get_length('spacing');
-            this._windowList.layout_manager.spacing = spacing;
-        });
         this._windowList.connect('scroll-event', this._onScrollEvent.bind(this));
 
         let indicatorsBox = new St.BoxLayout({x_align: Clutter.ActorAlign.END});
         box.add_child(indicatorsBox);
 
-        this._workspaceIndicator = new WorkspaceIndicator();
+        this._workspaceIndicator = new BottomWorkspaceIndicator({
+            baseStyleClass: 'window-list-workspace-indicator',
+            settings,
+        });
         indicatorsBox.add_child(this._workspaceIndicator.container);
 
         this._mutterSettings = new Gio.Settings({schema_id: 'org.gnome.mutter'});
@@ -763,7 +777,9 @@ class WindowList extends St.Widget {
         this._updateWorkspaceIndicatorVisibility();
 
         this._menuManager = new PopupMenu.PopupMenuManager(this);
-        this._menuManager.addMenu(this._workspaceIndicator.menu);
+        this._workspaceIndicator.connectObject('menu-set',
+            () => this._onWorkspaceMenuSet(), this);
+        this._onWorkspaceMenuSet();
 
         Main.layoutManager.addChrome(this, {
             affectsStruts: true,
@@ -822,7 +838,7 @@ class WindowList extends St.Widget {
 
         this._windowSignals = new Map();
         global.display.connectObject(
-            'window-created', (dsp, win) => this._addWindow(win), this);
+            'window-created', (dsp, win) => this._addWindow(win, true), this);
 
         Main.xdndHandler.connectObject(
             'drag-begin', () => this._monitorDrag(),
@@ -860,6 +876,11 @@ class WindowList extends St.Widget {
         children[newActive].activate();
     }
 
+    _onWorkspaceMenuSet() {
+        if (this._workspaceIndicator.menu)
+            this._menuManager.addMenu(this._workspaceIndicator.menu);
+    }
+
     _updatePosition() {
         this.set_position(
             this._monitor.x,
@@ -934,14 +955,14 @@ class WindowList extends St.Widget {
                        w2.metaWindow.get_stable_sequence();
             });
             for (let i = 0; i < windows.length; i++)
-                this._addWindow(windows[i].metaWindow);
+                this._addWindow(windows[i].metaWindow, false);
         } else {
             let apps = this._appSystem.get_running().sort((a1, a2) => {
                 return _getAppStableSequence(a1) -
                        _getAppStableSequence(a2);
             });
             for (let i = 0; i < apps.length; i++)
-                this._addApp(apps[i]);
+                this._addApp(apps[i], false);
         }
     }
 
@@ -955,26 +976,26 @@ class WindowList extends St.Widget {
             return;
 
         if (app.state === Shell.AppState.RUNNING)
-            this._addApp(app);
+            this._addApp(app, true);
         else if (app.state === Shell.AppState.STOPPED)
             this._removeApp(app);
     }
 
-    _addApp(app) {
+    _addApp(app, animate) {
         let button = new AppButton(app, this._perMonitor, this._monitor.index);
         this._settings.bind('display-all-workspaces',
             button, 'ignore-workspace', Gio.SettingsBindFlags.GET);
         this._windowList.add_child(button);
+        button.show(animate);
     }
 
     _removeApp(app) {
         let children = this._windowList.get_children();
         let child = children.find(c => c.app === app);
-        if (child)
-            child.destroy();
+        child?.animateOutAndDestroy();
     }
 
-    _addWindow(win) {
+    _addWindow(win, animate) {
         if (!this._grouped)
             this._checkGrouping();
 
@@ -992,6 +1013,7 @@ class WindowList extends St.Widget {
         this._settings.bind('display-all-workspaces',
             button, 'ignore-workspace', Gio.SettingsBindFlags.GET);
         this._windowList.add_child(button);
+        button.show(animate);
     }
 
     _removeWindow(win) {
@@ -1008,8 +1030,7 @@ class WindowList extends St.Widget {
 
         let children = this._windowList.get_children();
         let child = children.find(c => c.metaWindow === win);
-        if (child)
-            child.destroy();
+        child?.animateOutAndDestroy();
     }
 
     _monitorDrag() {
@@ -1080,6 +1101,28 @@ class WindowList extends St.Widget {
     }
 }
 
+class BottomWorkspaceIndicator extends WorkspaceIndicator {
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(params) {
+        super(params);
+
+        this.remove_style_class_name('panel-button');
+    }
+
+    setMenu(menu) {
+        super.setMenu(menu);
+
+        if (!menu)
+            return;
+
+        this.menu.actor.updateArrowSide(St.Side.BOTTOM);
+        this.menu.actor.remove_style_class_name('panel-menu');
+    }
+}
+
 export default class WindowListExtension extends Extension {
     constructor(metadata) {
         super(metadata);
@@ -1131,67 +1174,3 @@ export default class WindowListExtension extends Extension {
         return this._windowLists.some(list => list.contains(actor));
     }
 }
-
-class Tooltip extends St.Label {
-    static {
-        GObject.registerClass(this);
-    }
-
-    constructor(widget, params) {
-        super(params);
-
-        this._widget = widget;
-
-        this._widget.connect('notify::hover', () => {
-            if (this._widget.hover)
-                this.open();
-            else
-                this.close();
-        });
-    }
-
-    open() {
-        const buttonTitleWidget = this._widget.label_actor;
-        const [, , preferredTitleWidth] = buttonTitleWidget.get_preferred_size();
-        const maxTitleWidth = buttonTitleWidget.allocation.get_width();
-        const isTitleFullyShown = preferredTitleWidth <= maxTitleWidth;
-
-        if (isTitleFullyShown)
-            return;
-
-        this.set({
-            text: this._widget.label_actor.get_text(),
-            visible: true,
-            opacity: 0,
-        });
-
-        const [stageX, stageY] = this._widget.get_transformed_position();
-        const thumbWidth = this._widget.allocation.get_width();
-        const tipWidth = this.width;
-        const tipHeight = this.height;
-        const xOffset = Math.floor((thumbWidth - tipWidth) / 2);
-        const monitor = Main.layoutManager.findMonitorForActor(this);
-        const x = Math.clamp(
-            stageX + xOffset,
-            monitor.x,
-            monitor.x + monitor.width - tipWidth);
-        const y = stageY - tipHeight - TOOLTIP_OFFSET;
-        this.set_position(x, y);
-
-        this.ease({
-            opacity: 255,
-            duration: TOOLTIP_ANIMATION_TIME,
-            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-            onComplete: () => (this.visible = this._widget.hover),
-        });
-    }
-
-    close() {
-        this.ease({
-            opacity: 0,
-            duration: TOOLTIP_ANIMATION_TIME,
-            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-            onComplete: () => (this.visible = this._widget.hover),
-        });
-    }
-}
